searchData={"items":[{"type":"module","title":"CSV","doc":"RFC 4180 compliant CSV parsing and encoding for Elixir. Allows to specify\nother separators, so it could also be named: TSV, but it isn't.","ref":"CSV.html"},{"type":"function","title":"CSV.decode/2","doc":"Decode a stream of comma-separated lines into a stream of tuples. Decoding\nerrors will be inlined into the stream.","ref":"CSV.html#decode/2"},{"type":"function","title":"Options - CSV.decode/2","doc":"These are the options:\n\n* `:separator`            – The separator token to use, defaults to `?,`.\n    Must be a codepoint (syntax: ? + (your separator)).\n* `:escape_character`     – The escape character token to use, defaults to `?\"`.\n    Must be a codepoint (syntax: ? + (your escape character)).\n* `:escape_max_lines`     – The number of lines an escape sequence is allowed\n    to span, defaults to 10.\n* `:field_transform`      – A function with arity 1 that will get called with\n    each field and can apply transformations. Defaults to identity function.\n    This function will get called for every field and therefore should return\n    quickly.\n* `:headers`              – When set to `true`, will take the first row of\n    the csv and use it as header values.\n    When set to a list, will use the given list as header values.\n    When set to `false` (default), will use no header values.\n    When set to anything but `false`, the resulting rows in the matrix will\n    be maps instead of lists.\n* `:validate_row_length`  – When set to `true`, will take the first row of\n    the csv or its headers and validate that following rows are of the same\n    length. Defaults to `false`.\n* `:unescape_formulas`    – When set to `true`, will remove formula escaping\n    inserted to prevent [CSV Injection](https://owasp.org/www-community/attacks/CSV_Injection).\n* `:redact_errors`        – When set to `true`, will redact csv data in\n    message output of errors. This is to address scenarios where errors are\n    propagated in a stream and accidental logging of sensitive data needs\n    to be prevented. Defaults to `false`.","ref":"CSV.html#decode/2-options"},{"type":"function","title":"Examples - CSV.decode/2","doc":"Convert a filestream into a stream of rows in order of the given stream:\n\n    iex> \"../test/fixtures/docs/valid.csv\"\n    iex> |> Path.expand(__DIR__)\n    iex> |> File.stream!\n    iex> |> CSV.decode\n    iex> |> Enum.take(2)\n    [ok: [\"a\",\"b\",\"c\"], ok: [\"d\",\"e\",\"f\"]]\n\nRead from a file with a Byte Order Mark (BOM):\n\n    iex> \"../test/fixtures/utf8-with-bom.csv\"\n    ...> |> Path.expand(__DIR__)\n    ...> |> File.stream!([:trim_bom])\n    ...> |> CSV.decode()\n    ...> |> Enum.take(2)\n    [ok: [\"a\", \"b\"], ok: [\"d\", \"e\"]]\n\nRead from an UTF-16 encoded file with a Byte Order Mark (BOM):\n\n    iex> \"../test/fixtures/utf16-little-with-bom.csv\"\n    ...> |> Path.expand(__DIR__)\n    ...> |> File.stream!([:trim_bom, encoding: {:utf16, :little}])\n    ...> |> CSV.decode()\n    ...> |> Enum.take(3)\n    [ok: [\"a\", \"b\", \"c\"], ok: [\"1\", \"2\", \"3\"], ok: [\"4\", \"5\", \"ʤ\"]]\n\nRead from an UTF-16 big endian encoded file with a Byte Order Mark (BOM):\n\n    iex> \"../test/fixtures/utf16-big-with-bom.csv\"\n    ...> |> Path.expand(__DIR__)\n    ...> |> File.stream!([:trim_bom, encoding: {:utf16, :big}])\n    ...> |> CSV.decode()\n    ...> |> Enum.take(3)\n    [ok: [\"a\", \"b\", \"c\"], ok: [\"1\", \"2\", \"3\"], ok: [\"4\", \"5\", \"ʤ\"]]\n\nErrors will show up as error tuples:\n\n    iex> \"../test/fixtures/docs/escape-errors.csv\"\n    iex> |> Path.expand(__DIR__)\n    iex> |> File.stream!\n    iex> |> CSV.decode\n    iex> |> Enum.take(2)\n    [\n      ok: [\"a\",\"b\",\"c\"],\n      error: \"Escape sequence started on line 2:\\n\\n\\\"d,e,f\\n\\ndid not terminate before the stream halted. Parsing will continue on line 3.\\n\"\n    ]\n\nMap an existing stream of lines separated by a token to a stream of rows\nwith a header row:\n\n    iex> [\"a;b\\n\",\"c;d\\n\", \"e;f\\n\"]\n    iex> |> Stream.map(&(&1))\n    iex> |> CSV.decode(separator: ?;, headers: true)\n    iex> |> Enum.take(2)\n    [\n      ok: %{\"a\" => \"c\", \"b\" => \"d\"},\n      ok: %{\"a\" => \"e\", \"b\" => \"f\"}\n    ]\n\nMap a stream with custom escape characters:\n\n    iex> [\"@a@,@b@\\n\",\"@c@,@d@\\n\"]\n    ...> |> Stream.map(&(&1))\n    ...> |> CSV.decode(escape_character: ?@)\n    ...> |> Enum.take(2)\n    [ok: [\"a\", \"b\"], ok: [\"c\", \"d\"]]\n\nMap a stream with custom separator characters:\n\n    iex> [\"a;b\\n\",\"c;d\\n\"]\n    ...> |> Stream.map(&(&1))\n    ...> |> CSV.decode(separator: ?;)\n    ...> |> Enum.take(2)\n    [ok: [\"a\", \"b\"], ok: [\"c\", \"d\"]]\n\nTrim each field:\n\n    iex> [\" a , b   \\n\",\" c   ,   d \\n\"]\n    ...> |> Stream.map(&(&1))\n    ...> |> CSV.decode(field_transform: &String.trim/1)\n    ...> |> Enum.take(2)\n    [ok: [\"a\", \"b\"], ok: [\"c\", \"d\"]]\n\nMap an existing stream of lines separated by a token to a stream of rows\nwith a given header row:\n\n    iex> [\"a;b\\n\",\"c;d\\n\", \"e;f\\n\"]\n    iex> |> Stream.map(&(&1))\n    iex> |> CSV.decode(separator: ?;, headers: [:x, :y])\n    iex> |> Enum.take(2)\n    [\n      ok: %{:x => \"a\", :y => \"b\"},\n      ok: %{:x => \"c\", :y => \"d\"}\n    ]","ref":"CSV.html#decode/2-examples"},{"type":"function","title":"CSV.decode!/2","doc":"Decode a stream of comma-separated lines into a stream of tuples. Errors\nwhen decoding will get raised immediately.","ref":"CSV.html#decode!/2"},{"type":"function","title":"Options - CSV.decode!/2","doc":"These are the options:\n\n* `:separator`           – The separator token to use, defaults to `?,`.\n    Must be a codepoint (syntax: ? + (your separator)).\n* `:escape_character`    – The escape character token to use, defaults to `?\"`.\n    Must be a codepoint (syntax: ? + (your escape character)).\n* `:field_transform`     – A function with arity 1 that will get called with\n    each field and can apply transformations. Defaults to identity function.\n    This function will get called for every field and therefore should return\n    quickly.\n* `:headers`             – When set to `true`, will take the first row of\n    the csv and use it as header values.\n    When set to a list, will use the given list as header values.\n    When set to `false` (default), will use no header values.\n    When set to anything but `false`, the resulting rows in the matrix will\n    be maps instead of lists.\n* `:validate_row_length` – When set to `true`, will take the first row of\n    the csv or its headers and validate that following rows are of the same\n    length. Will raise an error if validation fails. Defaults to `false`.\n* `:unescape_formulas`   – When set to `true`, will remove formula escaping\n    inserted to prevent [CSV Injection](https://owasp.org/www-community/attacks/CSV_Injection).\n* `:unredact_exceptions` – When set to `true`, will show csv data in\n    message output of exceptions thrown. Only use this when using CSV strict \n    mode in environments and situations where there is no concern with \n    exception message data leaking in logs. Defaults to `false`.","ref":"CSV.html#decode!/2-options"},{"type":"function","title":"Examples - CSV.decode!/2","doc":"Convert a filestream into a stream of rows in order of the given stream:\n\n    iex> \"../test/fixtures/docs/valid.csv\"\n    iex> |> Path.expand(__DIR__)\n    iex> |> File.stream!()\n    iex> |> CSV.decode!()\n    iex> |> Enum.take(2)\n    [[\"a\",\"b\",\"c\"], [\"d\",\"e\",\"f\"]]\n\nRead from a file with a Byte Order Mark (BOM):\n\n    iex> \"../test/fixtures/utf8-with-bom.csv\"\n    ...> |> Path.expand(__DIR__)\n    ...> |> File.stream!([:trim_bom])\n    ...> |> CSV.decode!()\n    ...> |> Enum.take(2)\n    [[\"a\", \"b\"], [\"d\", \"e\"]]\n\nRead from an UTF-16 encoded file with a Byte Order Mark (BOM):\n\n    iex> \"../test/fixtures/utf16-little-with-bom.csv\"\n    ...> |> Path.expand(__DIR__)\n    ...> |> File.stream!([:trim_bom, encoding: {:utf16, :little}])\n    ...> |> CSV.decode!()\n    ...> |> Enum.take(3)\n    [[\"a\", \"b\", \"c\"], [\"1\", \"2\", \"3\"], [\"4\", \"5\", \"ʤ\"]]\n\nRead from an UTF-16 big endian encoded file with a Byte Order Mark (BOM):\n\n    iex> \"../test/fixtures/utf16-big-with-bom.csv\"\n    ...> |> Path.expand(__DIR__)\n    ...> |> File.stream!([:trim_bom, encoding: {:utf16, :big}])\n    ...> |> CSV.decode!()\n    ...> |> Enum.take(3)\n    [[\"a\", \"b\", \"c\"], [\"1\", \"2\", \"3\"], [\"4\", \"5\", \"ʤ\"]]\n\nMap an existing stream of lines separated by a token to a stream of rows\nwith a header row:\n\n    iex> [\"a;b\\n\",\"c;d\\n\", \"e;f\"]\n    iex> |> Stream.map(&(&1))\n    iex> |> CSV.decode!(separator: ?;, headers: true)\n    iex> |> Enum.take(2)\n    [\n      %{\"a\" => \"c\", \"b\" => \"d\"},\n      %{\"a\" => \"e\", \"b\" => \"f\"}\n    ]\n\nMap a stream with custom escape characters:\n\n    iex> [\"@a@,@b@\\n\",\"@c@,@d@\\n\"]\n    ...> |> Stream.map(&(&1))\n    ...> |> CSV.decode!(escape_character: ?@)\n    ...> |> Enum.take(2)\n    [[\"a\", \"b\"], [\"c\", \"d\"]]\n\nMap an existing stream of lines separated by a token to a stream of rows\nwith a given header row:\n\n    iex> [\"a;b\\n\",\"c;d\\n\", \"e;f\"]\n    iex> |> Stream.map(&(&1))\n    iex> |> CSV.decode!(separator: ?;, headers: [:x, :y])\n    iex> |> Enum.take(2)\n    [\n      %{:x => \"a\", :y => \"b\"},\n      %{:x => \"c\", :y => \"d\"}\n    ]\n\nTrim each field:\n\n    iex> [\" a , b   \\n\",\" c   ,   d \\n\"]\n    ...> |> Stream.map(&(&1))\n    ...> |> CSV.decode!(field_transform: &String.trim/1)\n    ...> |> Enum.take(2)\n    [[\"a\", \"b\"], [\"c\", \"d\"]]\n\nReplace invalid codepoints:\n\n    iex> \"../test/fixtures/broken-encoding.csv\"\n    ...> |> Path.expand(__DIR__)\n    ...> |> File.stream!()\n    ...> |> CSV.decode!(field_transform: fn field ->\n    ...>   if String.valid?(field) do\n    ...>     field\n    ...>   else\n    ...>     field\n    ...>     |> String.codepoints()\n    ...>     |> Enum.map(fn codepoint -> if String.valid?(codepoint), do: codepoint, else: \"?\" end)\n    ...>     |> Enum.join()\n    ...>   end\n    ...> end)\n    ...> |> Enum.take(2)\n    [[\"a\", \"b\", \"c\", \"?_?\"], [\"ಠ_ಠ\"]]","ref":"CSV.html#decode!/2-examples"},{"type":"function","title":"CSV.encode/2","doc":"Encode a table stream into a stream of RFC 4180 compliant CSV lines for\nwriting to a file or other IO.","ref":"CSV.html#encode/2"},{"type":"function","title":"Options - CSV.encode/2","doc":"These are the options:\n\n  * `:separator`              – The separator token to use, defaults to `?,`.\n  Must be a codepoint (syntax: ? + (your separator)).\n  * `:escape_character`       – The escape character token to use, defaults to `?\"`.\n  Must be a codepoint (syntax: ? + (your escape character)).\n  * `:delimiter`              – The delimiter token to use, defaults to `\\r\\n`.\n  Must be a string.\n  * `:force_escaping`          – When set to `true`, will escape fields even if\n  they do not contain characters that require escaping\n  * `:escape_formulas`         – When set to `true`, will escape formulas\n  to prevent [CSV Injection](https://owasp.org/www-community/attacks/CSV_Injection).\n  * `:headers`\n      * When set to `false` (default), will use the raw inputs as elements.  When set to anything but `false`, all elements in the input stream are assumed to be maps.\n      * When set to `true`, uses the keys of the first map as the first\n  element in the stream. All subsequent elements are the values of the maps.\n      * When set to a list, will use the given list as the first element\n      in the stream and order all subsequent elements using that list.\n      * When set to a keyword list, will use the keys of the\n      keyword list to match the keys of the data, and the values of the\n      keyword list to be the values in the first row of the output.","ref":"CSV.html#encode/2-options"},{"type":"function","title":"Examples - CSV.encode/2","doc":"Convert a stream of rows with fields into a stream of lines:\n\n    iex> [~w(a b), ~w(c d)]\n    iex> |> CSV.encode\n    iex> |> Enum.take(2)\n    [\"a,b\\r\\n\", \"c,d\\r\\n\"]\n\nConvert a stream of rows with fields with escape sequences into a stream of\nlines:\n\n    iex> [[\"a\\nb\", \"\\tc\"], [\"de\", \"\\tf\\\"\"]]\n    iex> |> CSV.encode(separator: ?\\t, delimiter: \"\\n\")\n    iex> |> Enum.take(2)\n    [\"\\\"a\\nb\\\"\\t\\\"\\tc\\\"\\n\", \"de\\t\\\"\\tf\\\"\\\"\\\"\\n\"]\n\nConvert a stream of rows with fields into a stream of lines forcing escaping\nwith a custom character:\n\n    iex> [~w(a b), ~w(c d)]\n    iex> |> CSV.encode(force_escaping: true, escape_character: ?@)\n    iex> |> Enum.take(2)\n    [\"@a@,@b@\\r\\n\", \"@c@,@d@\\r\\n\"]\n\nConvert a stream of rows with fields with formulas into a stream of\nlines:\n\n    iex> [~w(@a =b), ~w(-c +d)]\n    iex> |> CSV.encode(escape_formulas: true)\n    iex> |> Enum.take(2)\n    [\"\\\"'@a\\\",\\\"'=b\\\"\\r\\n\", \"\\\"'-c\\\",\\\"'+d\\\"\\r\\n\"]\n\nConvert a stream of row maps\n\n    iex> [%{header1: \"header1 value1\", header2: \"header2 value1\"}]\n    ...> |> CSV.encode(headers: true)\n    ...> |> Enum.to_list()\n    [\"header1,header2\\r\\n\", \"header1 value1,header2 value1\\r\\n\"]\n\nConvert a stream of rows renaming the headers by passing in a keyword list\n\n    iex> [%{a: \"value!\"}]\n    ...> |> CSV.encode(headers: [a: \"x\", b: \"y\"])\n    ...> |> Enum.to_list()\n    [\"x,y\\r\\n\", \"value!,\\r\\n\"]","ref":"CSV.html#encode/2-examples"},{"type":"type","title":"CSV.decode_options/0","doc":"","ref":"CSV.html#t:decode_options/0"},{"type":"type","title":"CSV.encode_options/0","doc":"","ref":"CSV.html#t:encode_options/0"},{"type":"module","title":"CSV.Decoding.Decoder","doc":"The Decoder CSV module sends lines of delimited values from a stream to the\nparser and converts rows coming from the CSV parser module to a consumable\nstream.","ref":"CSV.Decoding.Decoder.html"},{"type":"function","title":"CSV.Decoding.Decoder.decode/2","doc":"Decode a stream of comma-separated lines into a stream of rows that are\neither lists of fields or maps of headers to fields.\nThe Decoder expects line or variable size byte stream input.","ref":"CSV.Decoding.Decoder.html#decode/2"},{"type":"function","title":"Options - CSV.Decoding.Decoder.decode/2","doc":"These are the options:\n\n* `:separator`           – The separator token to use, defaults to `?,`.\n    Must be a codepoint (syntax: ? + (your separator)).\n* `:escape_character`    – The escape character token to use, defaults to `?\"`.\n    Must be a codepoint (syntax: ? + (your escape character)).\n* `:escape_max_lines`    – The number of lines an escape sequence is allowed\n    to span, defaults to 10.\n* `:field_transform`     – A function with arity 1 that will get called with\n    each field and can apply transformations. Defaults to identity function.\n    This function will get called for every field and therefore should return\n    quickly.\n* `:headers`             – When set to `true`, will take the first row of\n    the csv and use it as header values.\n    When set to a list, will use the given list as header values.\n    When set to `false` (default), will use no header values.\n    When set to anything but `false`, the resulting rows in the matrix will\n    be maps instead of lists.\n* `:validate_row_length` – When set to `true`, will take the first row of\n    the csv or its headers and validate that following rows are of the same\n    length. Defaults to `false`.\n* `:unescape_formulas`   – When set to `true`, will remove formula escaping\n    inserted to prevent [CSV Injection](https://owasp.org/www-community/attacks/CSV_Injection).","ref":"CSV.Decoding.Decoder.html#decode/2-options"},{"type":"function","title":"Examples - CSV.Decoding.Decoder.decode/2","doc":"Convert a stream with inlined escape sequences into a stream of rows:\n\n    iex> [\"a,b\\n\",\"c,d\\n\"]\n    ...> |> Stream.map(&(&1))\n    ...> |> CSV.Decoding.Decoder.decode\n    ...> |> Enum.take(2)\n    [ok: [\"a\", \"b\"], ok: [\"c\", \"d\"]]\n\nConvert a stream with custom escape characters into a stream of rows:\n\n    iex> [\"@a@,@b@\\n\",\"@c@,@d@\\n\"]\n    ...> |> Stream.map(&(&1))\n    ...> |> CSV.Decoding.Decoder.decode(escape_character: ?@)\n    ...> |> Enum.take(2)\n    [ok: [\"a\", \"b\"], ok: [\"c\", \"d\"]]\n\nConvert a line stream with escape sequences into a stream of rows:\n\n    iex> [\"'@a,'=b\\n\",\"'-c,'+d\\n\"]\n    ...> |> Stream.map(&(&1))\n    ...> |> CSV.Decoding.Decoder.decode(unescape_formulas: true)\n    ...> |> Enum.take(2)\n    [ok: [\"@a\", \"=b\"], ok: [\"-c\", \"+d\"]]\n\nTrim each field:\n\n    iex> [\" a , b   \\n\",\" c   ,   d \\n\"]\n    ...> |> Stream.map(&(&1))\n    ...> |> CSV.Decoding.Decoder.decode(field_transform: &String.trim/1)\n    ...> |> Enum.take(2)\n    [ok: [\"a\", \"b\"], ok: [\"c\", \"d\"]]\n\nRead from a file with a Byte Order Mark (BOM):\n\n    iex> \"../../../test/fixtures/utf8-with-bom.csv\"\n    ...> |> Path.expand(__DIR__)\n    ...> |> File.stream!([:trim_bom])\n    ...> |> CSV.Decoding.Decoder.decode()\n    ...> |> Enum.take(2)\n    [ok: [\"a\", \"b\"], ok: [\"d\", \"e\"]]\n\nReplace invalid codepoints:\n\n    iex> \"../../../test/fixtures/broken-encoding.csv\"\n    ...> |> Path.expand(__DIR__)\n    ...> |> File.stream!()\n    ...> |> CSV.Decoding.Decoder.decode(field_transform: fn field ->\n    ...>   if String.valid?(field) do\n    ...>     field\n    ...>   else\n    ...>     field\n    ...>     |> String.codepoints()\n    ...>     |> Enum.map(fn codepoint -> if String.valid?(codepoint), do: codepoint, else: \"?\" end)\n    ...>     |> Enum.join()\n    ...>   end\n    ...> end)\n    ...> |> Enum.take(2)\n    [ok: [\"a\", \"b\", \"c\", \"?_?\"], ok: [\"ಠ_ಠ\"]]\n\nMap an existing stream of lines separated by a token to a stream of rows with\na header row:\n\n    iex> [\"a;b\\n\",\"c;d\\n\", \"e;f\\n\"]\n    ...> |> Stream.map(&(&1))\n    ...> |> CSV.Decoding.Decoder.decode(separator: ?;, headers: true)\n    ...> |> Enum.take(2)\n    [\n      ok: %{\"a\" => \"c\", \"b\" => \"d\"},\n      ok: %{\"a\" => \"e\", \"b\" => \"f\"}\n    ]\n\nMap an existing stream of lines separated by a token to a stream of rows with\na header row with duplications:\n\n    iex> [\"a;b;b\\n\",\"c;d;e\\n\", \"f;g;h\\n\"]\n    ...> |> Stream.map(&(&1))\n    ...> |> CSV.Decoding.Decoder.decode(separator: ?;, headers: true)\n    ...> |> Enum.take(2)\n    [\n      ok: %{\"a\" => \"c\", \"b\" => [\"d\", \"e\"]},\n      ok: %{\"a\" => \"f\", \"b\" => [\"g\", \"h\"]}\n    ]\n\nMap an existing stream of lines separated by a token to a stream of rows\nwith a given header row:\n\n    iex> [\"a;b\\n\",\"c;d\\n\", \"e;f\\n\"]\n    ...> |> Stream.map(&(&1))\n    ...> |> CSV.Decoding.Decoder.decode(separator: ?;, headers: [:x, :y])\n    ...> |> Enum.take(2)\n    [\n      ok: %{:x => \"a\", :y => \"b\"},\n      ok: %{:x => \"c\", :y => \"d\"}\n    ]\n\nDecode a CSV string:\n\n    iex> [\"id,name\\r\\n1,Jane\\r\\n2,George\\r\\n3,John\"]\n    ...> |> CSV.Decoding.Decoder.decode(headers: true)\n    ...> |> Enum.map(&(&1))\n    [\n      ok: %{\"id\" => \"1\", \"name\" => \"Jane\"},\n      ok: %{\"id\" => \"2\", \"name\" => \"George\"},\n      ok: %{\"id\" => \"3\", \"name\" => \"John\"}\n    ]","ref":"CSV.Decoding.Decoder.html#decode/2-examples"},{"type":"type","title":"CSV.Decoding.Decoder.decode_options/0","doc":"","ref":"CSV.Decoding.Decoder.html#t:decode_options/0"},{"type":"module","title":"CSV.Decoding.Parser","doc":"The Parser CSV module transforms a stream of byte chunks or bytes\ninto a stream of row tuples and (potentially) error tuples. It\nfollows the grammar defined in [RFC4180](https://www.rfc-editor.org/rfc/rfc4180).","ref":"CSV.Decoding.Parser.html"},{"type":"function","title":"CSV.Decoding.Parser.add_error/3","doc":"","ref":"CSV.Decoding.Parser.html#add_error/3"},{"type":"function","title":"CSV.Decoding.Parser.add_row/2","doc":"","ref":"CSV.Decoding.Parser.html#add_row/2"},{"type":"function","title":"CSV.Decoding.Parser.parse/2","doc":"Parse a stream of comma-separated lines into a stream of rows.\nThe Parser expects line or variable size byte stream input.","ref":"CSV.Decoding.Parser.html#parse/2"},{"type":"function","title":"Options - CSV.Decoding.Parser.parse/2","doc":"These are the options:\n\n* `:separator`           – The separator token to use, defaults to `?,`.\n    Must be a codepoint (syntax: ? + (your separator)).\n* `:field_transform`     – A function with arity 1 that will get called with\n    each field and can apply transformations. Defaults to identity function.\n    This function will get called for every field and therefore should return\n    quickly.\n* `:unescape_formulas`   – When set to `true`, will remove formula escaping\n    inserted to prevent [CSV Injection](https://owasp.org/www-community/attacks/CSV_Injection).","ref":"CSV.Decoding.Parser.html#parse/2-options"},{"type":"function","title":"Examples - CSV.Decoding.Parser.parse/2","doc":"Convert a stream of lines with inlined escape sequences into a stream of rows:\n\n    iex> [\"a,b\\n\",\"c,d\\n\"]\n    ...> |> Stream.map(&(&1))\n    ...> |> CSV.Decoding.Parser.parse\n    ...> |> Enum.take(2)\n    [ok: [\"a\", \"b\"], ok: [\"c\", \"d\"]]\n\nConvert a stream of lines with into a stream of rows trimming each field:\n\n    iex> [\" a , b   \\n\",\" c   ,   d \\n\"]\n    ...> |> Stream.map(&(&1))\n    ...> |> CSV.Decoding.Parser.parse(field_transform: &String.trim/1)\n    ...> |> Enum.take(2)\n    [ok: [\"a\", \"b\"], ok: [\"c\", \"d\"]]","ref":"CSV.Decoding.Parser.html#parse/2-examples"},{"type":"type","title":"CSV.Decoding.Parser.parse_options/0","doc":"","ref":"CSV.Decoding.Parser.html#t:parse_options/0"},{"type":"module","title":"CSV.Defaults","doc":"The module defaults of CSV.","ref":"CSV.Defaults.html"},{"type":"protocol","title":"CSV.Encode","doc":"Implement encoding for your data types.","ref":"CSV.Encode.html"},{"type":"function","title":"CSV.Encode.encode/2","doc":"The encode function to implement, gets passed the data and env as a keyword\nlist containing the currently used separator and delimiter.","ref":"CSV.Encode.html#encode/2"},{"type":"type","title":"CSV.Encode.t/0","doc":"All the types that implement this protocol.","ref":"CSV.Encode.html#t:t/0"},{"type":"module","title":"CSV.Encoding.Encoder","doc":"The Encoder CSV module takes a table stream and transforms it into RFC 4180\ncompliant stream of lines for writing to a CSV File or other IO.","ref":"CSV.Encoding.Encoder.html"},{"type":"function","title":"CSV.Encoding.Encoder.encode/2","doc":"Encode a table stream into a stream of RFC 4180 compliant CSV lines for\nwriting to a file or other IO.","ref":"CSV.Encoding.Encoder.html#encode/2"},{"type":"function","title":"Options - CSV.Encoding.Encoder.encode/2","doc":"These are the options:\n\n  * `:separator`        – The separator character to use, defaults to `?,`.\n    Must be a codepoint (syntax: ? + your separator).\n  * `:escape_character` – The escape character to use, defaults to `?\"`.\n    Must be a codepoint (syntax: ? + your escape character).\n  * `:delimiter`        – The delimiter token to use, defaults to `\"\\r\\n\"`.\n  * `:headers`\n      * When set to `false` (default), will use the raw inputs as elements.  When set to anything but `false`, all elements in the input stream are assumed to be maps.\n      * When set to `true`, uses the keys of the first map as the first\n  element in the stream. All subsequent elements are the values of the maps.\n      * When set to a list, will use the given list as the first element\n      in the stream and order all subsequent elements using that list.\n      * When set to a keyword list, will use the keys of the\n      keyword list to match the keys of the data, and the values of the\n      keyword list to be the values in the first row of the output.\n  * `:escape_formulas`   – Escape formulas to prevent\n    [CSV Injection](https://owasp.org/www-community/attacks/CSV_Injection).","ref":"CSV.Encoding.Encoder.html#encode/2-options"},{"type":"function","title":"Examples - CSV.Encoding.Encoder.encode/2","doc":"Convert a stream of rows with cells into a stream of lines:\n\n    iex> [~w(a b), ~w(c d)]\n    iex> |> CSV.Encoding.Encoder.encode\n    iex> |> Enum.take(2)\n    [\"a,b\\r\\n\", \"c,d\\r\\n\"]\n\nConvert a stream of maps into a stream of lines:\n\n    iex> [%{\"a\" => 1, \"b\" => 2}, %{\"a\" => 3, \"b\" => 4}]\n    iex> |> CSV.Encoding.Encoder.encode(headers: true)\n    iex> |> Enum.to_list()\n    [\"a,b\\r\\n\", \"1,2\\r\\n\", \"3,4\\r\\n\"]\n\nConvert a stream of rows with cells with escape sequences into a stream of\nlines:\n\n    iex> [[\"a\\nb\", \"\\tc\"], [\"de\", \"\\tf\\\"\"]]\n    iex> |> CSV.Encoding.Encoder.encode(separator: ?\\t, delimiter: \"\\n\")\n    iex> |> Enum.take(2)\n    [\"\\\"a\\nb\\\"\\t\\\"\\tc\\\"\\n\", \"de\\t\\\"\\tf\\\"\\\"\\\"\\n\"]","ref":"CSV.Encoding.Encoder.html#encode/2-examples"},{"type":"exception","title":"CSV.EscapeSequenceError","doc":"Raised at runtime when the CSV stream either ends with unfinished escape sequences or\nescape sequences span more lines than specified by escape_max_lines (default 1000).","ref":"CSV.EscapeSequenceError.html"},{"type":"exception","title":"CSV.RowLengthError","doc":"Raised at runtime when the CSV has rows of variable length\nand `validate_row_length` is set to true.","ref":"CSV.RowLengthError.html"},{"type":"exception","title":"CSV.StrayEscapeCharacterError","doc":"Raised at runtime when the CSV row has stray quotes.","ref":"CSV.StrayEscapeCharacterError.html"},{"type":"extras","title":"Changelog","doc":"# Changelog","ref":"changelog.html"},{"type":"extras","title":"3.2.1 (2025-01-02) - Changelog","doc":"- Fix parser hanging when no newline finishes an unclosed escape sequence (reported by [@andrewtimberlake](https://github.com/andrewtimberlake) in [#132](https://github.com/beatrichartz/csv/issues/132) and fixed by [@kylewhite21](https://github.com/kylewhite21) in [#133](https://github.com/beatrichartz/csv/pull/133).\n- Fix `:unescape_formulas` removing characters even if they are not prefixing a formula. Fixed in [#131](https://github.com/beatrichartz/csv/pull/131) by [@jgmchan](https://github.com/jgmchan).","ref":"changelog.html#3-2-1-2025-01-02"},{"type":"extras","title":"3.2.1 (2023-11-26) - Changelog","doc":"- Correct typespec for decode, reported in [#125](https://github.com/beatrichartz/csv/issues/125) by [@AntoineAugusti](https://github.com/AntoineAugusti)","ref":"changelog.html#3-2-1-2023-11-26"},{"type":"extras","title":"3.2.0 (2023-09-24) - Changelog","doc":"- Strict mode: Exception messages of _thrown_ exceptions are now redacted by default to avoid data unintentionally leaking into logs.\n  This behaviour change is not considered to be breaking backwards compatibility since source data presented in exception messages is\n  not considered part of the `CSV` public API.\n- Strict mode: Exception messages can be unredacted using the `unredact_exceptions` option\n- Normal mode: Error messages can be redacted using the `redact_errors` option\n- Option to (un)redact exception messages [contributed in [#122](https://github.com/beatrichartz/csv/pull/124) by [@taylor-redden-papa](https://github.com/taylor-redden-papa)","ref":"changelog.html#3-2-0-2023-09-24"},{"type":"extras","title":"3.0.5 (2022-12-03) - Changelog","doc":"- Exclude dialyzer files from library package [contributed in [#121](https://github.com/beatrichartz/csv/pull/121) by [@milmazz](https://github.com/milmazz)","ref":"changelog.html#3-0-5-2022-12-03"},{"type":"extras","title":"3.0.4 (2022-11-19) - Changelog","doc":"- Add missing `escape_max_lines` to decode options typespec [closes #120](https://github.com/beatrichartz/csv/issues/120)","ref":"changelog.html#3-0-4-2022-11-19"},{"type":"extras","title":"3.0.3 (2022-11-04) - Changelog","doc":"- Ensure that reparsing of lines with stray escape characters does not produce duplicate error output [closes #119](https://github.com/beatrichartz/csv/issues/119)\n- Deduplication of type specs [in #118](https://github.com/beatrichartz/csv/pull/118) contributed by [@joseph-lozano](https://github.com/joseph-lozano)\n- Documentation fixes and improvements contributed by [@jamesvl](https://github.com/jamesvl) in [#115](https://github.com/beatrichartz/csv/pull/115)","ref":"changelog.html#3-0-3-2022-11-04"},{"type":"extras","title":"3.0.2 (2022-11-03) - Changelog","doc":"- Ensure that escaped fields as the last field on the last line without a newline are included in the results - [fixes #117](https://github.com/beatrichartz/csv/issues/117) raised by [@superhawk610](https://github.com/superhawk610)","ref":"changelog.html#3-0-2-2022-11-03"},{"type":"extras","title":"3.0.1 (2022-10-25) - Changelog","doc":"- Ensure that stray escape quotes and unterminated escape sequences on a last line without a newline produce errors","ref":"changelog.html#3-0-1-2022-10-25"},{"type":"extras","title":"3.0.0 (2022-10-25) - Changelog","doc":"- The parallel parser/lexer with a binary matching parser with better performance.\n- A new `:field_transform` option allows specifying functionality applied when decoding any field through a function\n- Escape characters can now be specified using the `:escape_character` option, this [Closes #59](https://github.com/beatrichartz/csv/issues/59)\n- The library will now reparse lines that follow e.g. an unterminated escape sequence. This ensures that all possible valid rows\n  will be returned in normal mode\n- Encoding checks have been removed because they can either be done using `:field_transform` or outside the library\n- Better docs\n\n#","ref":"changelog.html#3-0-0-2022-10-25"},{"type":"extras","title":"Upgrading from 2.x - Changelog","doc":"- **Parallelism has been removed**, alongside its options `:num_workers` and `:worker_work_ratio`. You can safely remove them.\n- **`StrayQuoteError` is now `StrayEscapeCharacterError`**. If you catch this error in your code, you need to rename it.\n- **The `:strip_fields` option needs to be replaced** with the `:field_transform` option:\n  ```elixir\n  File.stream!(\"data.csv\") |> CSV.decode(field_transform: &String.trim/1)\n  ```\n- **`:validate_row_length` now defaults to `false`**. This option produces an error for rows with different length. Set it\n  to `true` to get the same behaviour as in 2.x\n- **`:escape_formulas` is now `:unescape_formulas` for `decode` and `decode!`.** It is still `:escape_formulas` for\n  `encode`. Change `:escape_formulas` to `:unescape_formulas` in `decode` calls to get the same behaviour as in 2.x\n- **`:escape_max_lines` now defaults to `10`** instead of `1000`. To get the same behaviour as in 2.x, use:\n  ```elixir\n  File.stream!(\"data.csv\") |> CSV.decode(escape_max_lines: 1000)\n  ```\n- **`:replace` has been removed**. `CSV` will now return fields with incorrect encoding as-is. \n  You can use the new `:field_transform` option to provide a function transforming fields while they are being parsed. \n  This allows to e.g. replace incorrect encoding:\n  ```elixir\n  defp replace_bad_encoding(field) do\n    if String.valid?(field) do\n      field\n    else\n      field\n      |> String.codepoints()\n      |> Enum.map(fn codepoint -> if String.valid?(codepoint), do: codepoint, else: \"?\" end)\n      |> Enum.join()\n    end\n  end\n  ```","ref":"changelog.html#upgrading-from-2-x"},{"type":"extras","title":"2.5.0 (2022-09-17) - Changelog","doc":"- Optional parameter `escape_formulas` to prevent CSV injection. [Fixes #103](https://github.com/beatrichartz/csv/issues/103) reported by [@maennchen](https://github.com/maennchen). Contributed by [@maennchen](https://github.com/maennchen) in [PR #104](https://github.com/beatrichartz/csv/pull/104).\n- Optional parameter `force_quotes` to force quotes when encoding contributed by [@stuart](https://github.com/stuart)\n- Bugfix to pass non UTF-8 lines through in normal mode so other lines can be processed, [Fixes #107](https://github.com/beatrichartz/csv/pull/107). Contributed by [@al2o3cr](https://github.com/al2o3cr).\n- Allow to encode keyword lists specifying headers as values, contributed by [@michaelchu](https://github.com/michaelchu)\n- Better docs thanks to [@kianmeng](https://github.com/kianmeng)","ref":"changelog.html#2-5-0-2022-09-17"},{"type":"extras","title":"2.4.1 (2020-09-12) - Changelog","doc":"- Fix unnecessary escaping of delimiters when encoding [Fixes #70](https://github.com/beatrichartz/csv/issues/70)\n  reported by [@karmajunkie](https://github.com/karmajunkie)","ref":"changelog.html#2-4-1-2020-09-12"},{"type":"extras","title":"2.4.0 (2020-09-12) - Changelog","doc":"- Fix [StrayQuoteError](https://hexdocs.pm/csv/CSV.StrayQuoteError.html) not getting\n  passed the correct arguments in strict mode. [Fixes #96](https://github.com/beatrichartz/csv/issues/96).\n- When headers are present multiple times and the `:headers` option is set to `true`, parse the values into a list.\n  Contributed by [@MrAlexLau](https://github.com/MrAlexLau) in [PR #97](https://github.com/beatrichartz/csv/pull/97).","ref":"changelog.html#2-4-0-2020-09-12"},{"type":"extras","title":"2.3.1 (2019-03-30) - Changelog","doc":"- Fix [StrayQuoteError](https://hexdocs.pm/csv/CSV.StrayQuoteError.html) incorrectly\n  getting raised when escape sequences end in new lines. [Fixes #89](https://github.com/beatrichartz/csv/issues/89).\n  Raised by [@rockwood](https://github.com/rockwood) in [Issue #96](https://github.com/beatrichartz/csv/issues/96).","ref":"changelog.html#2-3-1-2019-03-30"},{"type":"extras","title":"2.3.0 (2019-03-17) - Changelog","doc":"- Add [StrayQuoteError](https://hexdocs.pm/csv/CSV.StrayQuoteError.html) which gets\n  raised when a row has stray quotes rather than [EscapeSequenceError](https://hexdocs.pm/csv/CSV.EscapeSequenceError.html#content)\n  to help with common encoding errors.","ref":"changelog.html#2-3-0-2019-03-17"},{"type":"extras","title":"2.2.0 (2019-03-03) - Changelog","doc":"- Make syntax compatible with latest Elixir releases\n- Add [`validate_row_length:` option](https://hexdocs.pm/csv/CSV.html#decode/2-options) defaulting to true to allow\n  disabling validation of row length.","ref":"changelog.html#2-2-0-2019-03-03"},{"type":"extras","title":"2.0.0 (2017-05-29) - Changelog","doc":"- Make [`decode`](https://hexdocs.pm/csv/CSV.html#decode/2) return row and\n  error tuples instead of raising errors directly\n- Make old behaviour of raising errors directly available\n  via [`decode!`](https://hexdocs.pm/csv/CSV.html#decode!/2)\n- Improve error messages for escape sequences\n- Rewrite parts of the pipeline to be more modular","ref":"changelog.html#2-0-0-2017-05-29"},{"type":"extras","title":"1.4.4 (2016-11-12) - Changelog","doc":"- Load [`parallel_stream`](https://github.com/beatrichartz/parallel_stream)\n  as an app dependency to avoid load level errors.\n  See [issue #56](https://github.com/beatrichartz/csv/issues/56) reported\n  by [@luk3thomas](https://github.com/luk3thomas)","ref":"changelog.html#1-4-4-2016-11-12"},{"type":"extras","title":"1.4.3 (2016-08-27) - Changelog","doc":"- Fix a case where lines would not be aggregated correctly\n  [see #52](https://github.com/beatrichartz/csv/issues/52) reported by\n  [@yury-dimov](https://github.com/yury-dymov)","ref":"changelog.html#1-4-3-2016-08-27"},{"type":"extras","title":"1.4.2 (2016-06-20) - Changelog","doc":"- Update dependency on [`parallel_stream`](https://github.com/beatrichartz/parallel_stream)","ref":"changelog.html#1-4-2-2016-06-20"},{"type":"extras","title":"1.4.1 (2016-05-21) - Changelog","doc":"- Fix condition where rows would be dropped when decoding from stateful streams.\n  [See #39](https://github.com/beatrichartz/csv/issues/39) reported by\n  [@moxley](https://github.com/moxley)","ref":"changelog.html#1-4-1-2016-05-21"},{"type":"extras","title":"1.4.0 (2016-04-03) - Changelog","doc":"- add option to specify headers in encode - added [in #34](https://github.com/beatrichartz/csv/issues/34)\n  by [@barruumrex](https://github.com/barruumrex)","ref":"changelog.html#1-4-0-2016-04-03"},{"type":"extras","title":"1.3.3 (2016-03-25) - Changelog","doc":"- Fix empty streams raising a lexer error - raised [in #28](https://github.com/beatrichartz/csv/issues/28)\n  by [@kiliancs](https://github.com/kiliancs)","ref":"changelog.html#1-3-3-2016-03-25"},{"type":"extras","title":"1.3.2 (2016-03-08) - Changelog","doc":"- Cleanup, removing some unused defaults in function headers to remove compile\n  time warnings","ref":"changelog.html#1-3-2-2016-03-08"},{"type":"extras","title":"1.3.1 (2016-03-08) - Changelog","doc":"- Fix `:strip_cells` not stripping cells when multiple options are specified - #29 by [@tomjoro](https://github.com/tomjoro)","ref":"changelog.html#1-3-1-2016-03-08"},{"type":"extras","title":"1.3.0 (2016-03-01) - Changelog","doc":"- Now supports linebreaks inside escaped fields (#13)\n- Raises an error when row length mismatches across rows\n- Uses [parallel_stream](https://github.com/beatrichartz/parallel_stream) for parallelism","ref":"changelog.html#1-3-0-2016-03-01"},{"type":"extras","title":"1.2.4 (2016-02-06) - Changelog","doc":"- Fix encoding of double quotes","ref":"changelog.html#1-2-4-2016-02-06"},{"type":"extras","title":"1.2.3 (2016-01-19) - Changelog","doc":"- Fix a condition where headers: true would enumerate the whole file once before parsing","ref":"changelog.html#1-2-3-2016-01-19"},{"type":"extras","title":"1.2.2 (2016-01-02) - Changelog","doc":"- Fix default num_pipes argument to evaluate num_pipes dependent on scheduler at runtime\n- Test utf-8 files with BOM\n- Syntax and mix updates for elixir 1.2","ref":"changelog.html#1-2-2-2016-01-02"},{"type":"extras","title":"1.2.1 (2015-10-17) - Changelog","doc":"- Decoder performance optimisations","ref":"changelog.html#1-2-1-2015-10-17"},{"type":"extras","title":"1.2.0 (2015-10-11) - Changelog","doc":"- Use `Stream.transform/4` - incompatible with Elixir < `1.1.0`","ref":"changelog.html#1-2-0-2015-10-11"},{"type":"extras","title":"1.1.5 (2015-10-11) - Changelog","doc":"- Decoder refactor from `Stream.resource/3` to `Stream.transform/3` in order to\n  get more predictable stream behaviour\n- Rows now get processed in order\n- Fix a bug where stream would get evaluated before being decoded","ref":"changelog.html#1-1-5-2015-10-11"},{"type":"extras","title":"1.1.4 (2015-09-13) - Changelog","doc":"- Fix a bug where headers could be out of order","ref":"changelog.html#1-1-4-2015-09-13"},{"type":"extras","title":"1.1.3 (2015-09-12) - Changelog","doc":"- Fix a bug where headers could get parsed as the first row","ref":"changelog.html#1-1-3-2015-09-12"},{"type":"extras","title":"1.1.2 (2015-09-05) - Changelog","doc":"- Fix a bug where calls to decode with num_pipes: 1 would yield varying\n  results due to leftover state in decoder message queue","ref":"changelog.html#1-1-2-2015-09-05"},{"type":"extras","title":"1.1.1 (2015-07-14) - Changelog","doc":"- Rescue from errors in stream producer to get more predictable behaviour\n  in case of failure","ref":"changelog.html#1-1-1-2015-07-14"},{"type":"extras","title":"1.1.0 (2015-07-12) - Changelog","doc":"- Better error messages when encountering invalid encodings","ref":"changelog.html#1-1-0-2015-07-12"},{"type":"extras","title":"1.0.1 (2015-07-11) - Changelog","doc":"- Indicate `consolidate_protocols` for better encoding performance","ref":"changelog.html#1-0-1-2015-07-11"},{"type":"extras","title":"1.0.0 (2015-05-24) - Changelog","doc":"- Use bytes as separators","ref":"changelog.html#1-0-0-2015-05-24"},{"type":"extras","title":"0.2.3 (2015-05-24) - Changelog","doc":"- Add benchmarking","ref":"changelog.html#0-2-3-2015-05-24"},{"type":"extras","title":"0.2.2 (2015-05-20) - Changelog","doc":"- Use utf-8 bytes instead of codepoints for multi-byte parsing","ref":"changelog.html#0-2-2-2015-05-20"},{"type":"extras","title":"0.2.1 (2015-05-20) - Changelog","doc":"- Fix handling of multi-byte utf-8 characters","ref":"changelog.html#0-2-1-2015-05-20"},{"type":"extras","title":"0.2.0 (2015-03-25) - Changelog","doc":"- Implement encoder protocol","ref":"changelog.html#0-2-0-2015-03-25"},{"type":"extras","title":"CSV","doc":"# CSV [![Build Status](https://github.com/beatrichartz/csv/actions/workflows/ci.yml/badge.svg?branch=main)](https://github.com/beatrichartz/csv) [![Coverage Status](https://coveralls.io/repos/github/beatrichartz/csv/badge.svg?branch=main)](https://coveralls.io/github/beatrichartz/csv?branch=main) [![Codacy Badge](https://app.codacy.com/project/badge/Grade/2b1154889a3f4d1681bf40a89834271c)](https://www.codacy.com/gh/beatrichartz/csv/dashboard?utm_source=github.com&amp;utm_medium=referral&amp;utm_content=beatrichartz/csv&amp;utm_campaign=Badge_Grade) [![Hex pm](http://img.shields.io/hexpm/v/csv.svg?style=flat)](https://hex.pm/packages/csv) [![Hex Docs](https://img.shields.io/badge/hex-docs-lightgreen.svg)](https://hexdocs.pm/csv/) [![License](https://img.shields.io/hexpm/l/csv.svg)](https://github.com/beatrichartz/csv/blob/main/LICENSE) [![Downloads](https://img.shields.io/hexpm/dw/csv.svg?style=flat)](https://hex.pm/packages/csv)\n\n[RFC 4180](http://tools.ietf.org/html/rfc4180) compliant, composable CSV parsing and encoding for Elixir.","ref":"readme.html"},{"type":"extras","title":"Installation - CSV","doc":"Add\n```elixir\n{:csv, \"~> 3.2\"}\n```\nto your deps in `mix.exs` like so:\n\n```elixir\ndefp deps do\n  [{:csv, \"~> 3.2\"}]\nend\n```","ref":"readme.html#installation"},{"type":"extras","title":"Getting all correctly formatted rows - CSV","doc":"CSV is a notoriously fickle format, with many implementations and files interpreting it differently.\n\nFor that reason, `CSV` implements a normal mode `CSV.decode` that will return a stream of `ok: [\"field1\", \"field2\"]`\nand `err: \"Message\"` tuples. It will also **reparse lines after a previous line has opened an unterminated escape sequence**,\nensuring you get all correctly formatted rows.\n\nThe goal of this library is to allow to extract all correctly formatted rows, while displaying descriptive errors for \nincorrectly formatted rows.\n\nIn strict mode using `CSV.decode!` the library will raise an exception when it encounters the first error, aborting the\noperation.","ref":"readme.html#getting-all-correctly-formatted-rows"},{"type":"extras","title":"Performance - CSV","doc":"This library uses fast binary matching and is able to parse about half a million rows of a moderately complex CSV file \nper second in a single process on a small cloud instance spec (2vCPU, 2GB Memory). CSV parsing will unlikely become a \nbottleneck in your data pipeline.\n\nIf you are reading from a large file, `CSV` will perform best when streaming with `:read_ahead` in byte mode:\n\n```elixir\nFile.stream!(\"data.csv\", [read_ahead: 100_000], 1000) |> CSV.decode()\n```\n\nWhile `1000` is usually a good default number of bytes to stream, you should measure performance and fine-tune\nbyte size according to your environment.","ref":"readme.html#performance"},{"type":"extras","title":"Upgrading from 2.x - CSV","doc":"The main goal for 3.x has been to streamline the library API and leverage binary matching. \n\n##","ref":"readme.html#upgrading-from-2-x"},{"type":"extras","title":"Upgrading should require few to no changes in most cases - CSV","doc":"- **Parallelism has been removed**, alongside its options `:num_workers` and `:worker_work_ratio`. You can safely remove them.\n- `CSV` now expects line breaks to be present in the data. If you used to parse strings by applying `String.split/2` before \n  passing it to decode, you can do the same now feeding in\n  the string as a single item of a list:\n  ```elixir\n  [\"a,b,c\\nd,e,f\"] |> CSV.decode()\n  ```\n- **`StrayQuoteError` is now `StrayEscapeCharacterError`**. If you catch this error in your code, you need to rename it.\n- **The `:strip_fields` option needs to be replaced** with the `:field_transform` option:\n  ```elixir\n  File.stream!(\"data.csv\") |> CSV.decode(field_transform: &String.trim/1)\n  ```\n- **`:validate_row_length` now defaults to `false`**. This option produces an error for rows with different length. Set it\n  to `true` to get the same behaviour as in 2.x\n- **`:escape_formulas` is now `:unescape_formulas` for `decode` and `decode!`.** It is still `:escape_formulas` for\n  `encode`. Change `:escape_formulas` to `:unescape_formulas` in `decode` calls to get the same behaviour as in 2.x\n- **`:escape_max_lines` now defaults to `10`** instead of `1000`. To get the same behaviour as in 2.x, use:\n  ```elixir\n  File.stream!(\"data.csv\") |> CSV.decode(escape_max_lines: 1000)\n  ```\n- **`:replace` has been removed**. `CSV` will now return fields with incorrect encoding as-is. \n  You can use the new `:field_transform` option to provide a function transforming fields while they are being parsed. \n  This allows to e.g. replace incorrect encoding:\n  ```elixir\n  defp replace_bad_encoding(field) do\n    if String.valid?(field) do\n      field\n    else\n      field\n      |> String.codepoints()\n      |> Enum.map(fn codepoint -> if String.valid?(codepoint), do: codepoint, else: \"?\" end)\n      |> Enum.join()\n    end\n  end\n\n  File.stream!(\"data.csv\") |> CSV.decode(field_transform: &replace_bad_encoding/1)\n  ```\n\n**That's it!** Please open an issue if you see any other non-backward compatible behaviour so it can be documented.\n\n#","ref":"readme.html#upgrading-should-require-few-to-no-changes-in-most-cases"},{"type":"extras","title":"Elixir version requirements - CSV","doc":"* Elixir `1.5.0` is required for all versions above `2.5.0`.\n* Elixir `1.1.0` is required for all versions above `1.1.5`.","ref":"readme.html#elixir-version-requirements"},{"type":"extras","title":"Design Goals - CSV","doc":"This library aims to to solve concerns related to csv parsing in data pipelines, following the UNIX philosophy:\nIt consumes streams or enumerables, producing streams of lists, maps or tuples depending on configuration. This simplifies\n using it in data pipelines, where CSV encoding or decoding is only one of the processing steps.","ref":"readme.html#design-goals"},{"type":"extras","title":"Usage - CSV","doc":"`CSV` can decode and encode from and to a stream of bytes or lines.\n\n#","ref":"readme.html#usage"},{"type":"extras","title":"Decoding - CSV","doc":"Do this to decode data:\n\n````elixir\n# Decode file line by line\nFile.stream!(\"data.csv\")\n  |> CSV.decode()\n\n# Decode a UTF-16 file with BOM\nFile.stream!([:trim_bom, encoding: {:utf16, :little}])\n  |> CSV.decode()\n\n# Decode file in chunks of 1000 bytes\nFile.stream!(\"data.csv\", [], 1000) \n  |> CSV.decode()\n\n# Decode a csv formatted string\n[\"long,csv,string\\\\nwith,multiple,lines\"] \n  |> CSV.decode()\n\n# Decode a list of arbitrarily chunked csv data\n[\"list,\", \"of,arbitrarily\", \"\\\\nchun\", \"ked,csv,data\\\\n\"] \n  |> CSV.decode()\n````\n\nAnd you'll get a stream of row tuples:\n````elixir\n[ok: [\"a\", \"b\"], ok: [\"c\", \"d\"]]\n````\n\nAnd, potentially error tuples:\n````elixir\n[error: \"\", ok: [\"c\", \"d\"]]\n````\n\nUse strict mode `decode!` to get a two-dimensional list, raising errors as they\noccur, aborting the operation:\n````elixir\nFile.stream!(\"data.csv\") |> CSV.decode!\n````\n\nUnredact source data in exceptions that `decode!` throws:\n```elixir\nFile.stream!(\"data.csv\") |> CSV.decode!(unredact_exceptions: true)\n```\n\n\n##","ref":"readme.html#decoding"},{"type":"extras","title":"Options - CSV","doc":"For all available options [check the docs on `decode`](https://hexdocs.pm/csv/CSV.html#decode/2)\n[and `decode!`](https://hexdocs.pm/csv/CSV.html#decode!/2)\n\nSpecify a semicolon separator:\n\n````elixir\nstream |> CSV.decode(separator: ?;)\n````\n\nSpecify a custom escape character:\n\n````elixir\nstream |> CSV.decode(escape_character: ?@)\n````\n\nApply a transformation to a field when parsed, e.g. trimming the field:\n\n````elixir\nstream |> CSV.decode(field_transform: &String.trim/1)\n````\n\nUnescape formulas that have been escaped:\n\n````elixir\nstream |> CSV.decode(unescape_formulas: true)\n````\n\nRedact source data in error tuples producted by decode:\n\n````elixir\nstream |> CSV.decode(redact_errors: true)\n````\n\n\n#","ref":"readme.html#options"},{"type":"extras","title":"Encoding - CSV","doc":"Do this to encode a table (two-dimensional list):\n\n````elixir\ntable_data |> CSV.encode\n````\n\nAnd you'll get a stream of lines ready to be written to an IO.\nSo, this is writing to a file:\n\n````elixir\nfile = File.open!(\"test.csv\", [:write, :utf8])\ntable_data |> CSV.encode |> Enum.each(&IO.write(file, &1))\n````\n\n##","ref":"readme.html#encoding"},{"type":"extras","title":"Options - CSV","doc":"Use a semicolon separator:\n\n````elixir\nyour_data |> CSV.encode(separator: ?;)\n````\n\nUse a specific escape character:\n\n````elixir\nyour_data |> CSV.encode(escape_character: ?@)\n````\n\nYou can also specify headers when encoding, which will encode map values into\nthe right place:\n\n````elixir\n[%{\"a\" => \"value!\"}] |> CSV.encode(headers: [\"z\", \"a\"])\n# [\"z,a\\\\r\\\\n\", \",value!\\\\r\\\\n\"]\n````\n\nYou can also specify a keyword list, the keys of the list will be used as the keys for the rows, \nbut the values will be the value used for the header row name in CSV output\n\n````elixir\n[%{a: \"value!\"}] |> CSV.encode(headers: [a: \"x\", b: \"y\"])\n# [\"x,y\\\\r\\\\n\", \"value!,\\\\r\\\\n\"]\n````\n\nYou'll surely appreciate some [more info on `encode`](https://hexdocs.pm/csv/CSV.html#encode/2).\n\n##","ref":"readme.html#options"},{"type":"extras","title":"Polymorphic encoding - CSV","doc":"Make sure your data gets encoded the way you want - implement the `CSV.Encode`\nprotocol for whatever you wish to encode:\n\n````elixir\ndefimpl CSV.Encode, for: MyData do\n  def encode(%MyData{has: fun}, env \\\\ []) do\n    \"so much #{fun}\" |> CSV.Encode.encode(env)\n  end\nend\n````\n\nOr similar.\n\n##","ref":"readme.html#polymorphic-encoding"},{"type":"extras","title":"Ensure performant encoding - CSV","doc":"The encoding protocol implements a fallback to Any for types where a simple call\no `to_string` will provide unambiguous results. Protocol dispatch for the\nfallback to Any is *very* slow when protocols are not consolidated, so make sure\nyou [have `consolidate_protocols: true`](http://blog.plataformatec.com.br/2015/04/build-embedded-and-start-permanent-in-elixir-1-0-4/)\nin your `mix.exs` or you consolidate protocols manually for production in order\nto get good performance.\n\nThere is more to know about everything :tm: - [Check the doc](http://hexdocs.pm/csv/)","ref":"readme.html#ensure-performant-encoding"},{"type":"extras","title":"Contributions & Bugfixes are most welcome! - CSV","doc":"Please make sure to add tests. I will not look at PRs that are\neither failing or lowering coverage. Also, solve one problem at\na time.","ref":"readme.html#contributions-bugfixes-are-most-welcome"},{"type":"extras","title":"Copyright and License - CSV","doc":"Copyright (c) 2022 Beat Richartz\n\nCSV source code is licensed under the [MIT License](https://github.com/beatrichartz/csv/blob/main/LICENSE).","ref":"readme.html#copyright-and-license"}],"content_type":"text/markdown"}